package `in`.finbox.bankconnect.finbox_bc_plugin

import `in`.finbox.bankconnect.hybrid.FinBoxBankConnect
import `in`.finbox.bankconnect.hybrid.app.BankConnectApp
import `in`.finbox.bankconnect.hybrid.payload.FinBoxPayLoad
import android.app.Activity
import android.content.Context
import android.content.Intent
import androidx.core.app.ActivityCompat.startActivityForResult
import com.google.gson.Gson
import com.google.gson.GsonBuilder
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.PluginRegistry
import java.util.*


class FinBoxBcPlugin : FlutterPlugin, MethodChannel.MethodCallHandler, ActivityAware {

    companion object {
        const val FINBOX_JOURNEY_RESULT = "finbox_result"
        fun initLibrary(context: Context) {
            BankConnectApp.initDi(context)
        }
    }

    private lateinit var bankConnectChannel: MethodChannel
    private var activity: Activity? = null
    private lateinit var context: Context
    override fun onAttachedToEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        context = binding.applicationContext
        bankConnectChannel =
            MethodChannel(binding.binaryMessenger, "in.finbox.bankconnect/hybrid")
        bankConnectChannel.setMethodCallHandler(this)
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        bankConnectChannel.setMethodCallHandler(null)
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        when (call.method) {
            "initSdk" -> {
                initSdk(
                    call.argument("apiKey")!!,
                    call.argument("fromDate")!!,
                    call.argument("toDate")!!,
                    call.argument("bankName")!!
                )
            }
        }
    }

    private fun initSdk(apiKey: String, fromDate: String, toDate: String, bankName: String) {
        val bankConnectBuilder = FinBoxBankConnect.Builder(context)
            .apiKey(apiKey)
            .linkId(UUID.randomUUID().toString())
            .fromDate(fromDate)
            .toDate(toDate)
            .bank(bankName)
            .build()
        this.activity?.let {
            startActivityForResult(
                it,
                bankConnectBuilder.getBankConnectIntent(context),
                123,
                null
            )
        }

    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        this.activity = binding.activity
        binding.addActivityResultListener(object : PluginRegistry.ActivityResultListener {
            override fun onActivityResult(
                requestCode: Int,
                resultCode: Int,
                data: Intent?
            ): Boolean {
                parseData(data)
                return true
            }

        })
    }

    override fun onDetachedFromActivityForConfigChanges() {
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    }

    private fun parseData(data: Intent?) {
        val result = data?.extras?.getParcelable<FinBoxPayLoad>(FINBOX_JOURNEY_RESULT)
        result?.let { finBoxPayLoad ->
            sendResult(finBoxPayLoad)
        } ?: sendResult(FinBoxPayLoad("","","Something went wrong",""))
    }

    override fun onDetachedFromActivity() {
        this.activity = null
    }

    private fun sendResult(data: FinBoxPayLoad): Unit {
        val gson = GsonBuilder().serializeNulls().create()
        bankConnectChannel.invokeMethod("getJourneyResult", gson.toJson(data))
    }


}